/*
 * Author: Daniel Nitecki
 * Date: 11-10-2012 23:59:34
 * 
 * Description:
 * This program generates the specific waveforms that control Syma helicopter movements.
 */

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "stm32f4_discovery.h"

/** @addtogroup STM32F4_Discovery_Peripheral_Examples
 * @{
 */

/** @addtogroup TIM_PWM_Output
 * @{
 */

/* Private typedef -----------------------------------------------------------*/
TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
TIM_OCInitTypeDef TIM_OCInitStructure;

/* Private define ------------------------------------------------------------*/
#define RAM_BASE 0x20000000
#define RAM_BB_BASE 0x22000000
#define Var_ResetBit_BB(VarAddr, BitNumber) (*(vu32 *) (RAM_BB_BASE | ((VarAddr - RAM_BASE) << 5) | ((BitNumber) << 2)) = 0)
#define Var_SetBit_BB(VarAddr, BitNumber) (*(vu32 *) (RAM_BB_BASE | ((VarAddr - RAM_BASE) << 5) | ((BitNumber) << 2)) = 1)
#define Var_GetBit_BB(VarAddr, BitNumber) (*(vu32 *) (RAM_BB_BASE | ((VarAddr - RAM_BASE) << 5) | ((BitNumber) << 2)))

#define varClrBit(var,bit) (Var_ResetBit_BB((u32)&var,bit))
#define varSetBit(var,bit) (Var_SetBit_BB((u32)&var,bit))
#define varGetBit(var,bit) (Var_GetBit_BB((u32)&var,bit))
//-------------

/* Private macro -------------------------------------------------------------*/

/* Private variables ---------------------------------------------------------*/
// Accelerometer
uint8_t Buffer[6];
__IO uint32_t TimingDelay = 0;
__IO int8_t XOffset;
__IO int8_t YOffset;
__IO int8_t ZOffset;

__IO uint8_t SingleClickDetect = 0x00;
extern uint8_t ClickReg;

void ThrottleLinkConfig(void);

/* Private functions ---------------------------------------------------------*/

/**
 * @brief  Main program
 * @param  None
 * @retval None
 */
int main(void)
{
	/*!< At this stage the microcontroller clock setting is already configured,
	 this is done through SystemInit() function which is called from startup
	 file (startup_stm32f4xx.s) before to branch to application main.
	 To reconfigure the default setting of SystemInit() function, refer to
	 system_stm32f4xx.c file
	 */
	  uint8_t ctrl = 0;
	  
	  LIS302DL_InitTypeDef  LIS302DL_InitStruct;
	  LIS302DL_InterruptConfigTypeDef LIS302DL_InterruptStruct;  
	  
	  /* SysTick end of count event each 10ms */
	  SysTick_Config(SystemCoreClock/100);
	  
	  /* Set configuration of LIS302DL*/
	  LIS302DL_InitStruct.Power_Mode = LIS302DL_LOWPOWERMODE_ACTIVE;
	  LIS302DL_InitStruct.Output_DataRate = LIS302DL_DATARATE_100;
	  LIS302DL_InitStruct.Axes_Enable = LIS302DL_X_ENABLE | LIS302DL_Y_ENABLE | LIS302DL_Z_ENABLE;
	  LIS302DL_InitStruct.Full_Scale = LIS302DL_FULLSCALE_2_3;
	  LIS302DL_InitStruct.Self_Test = LIS302DL_SELFTEST_NORMAL;
	  LIS302DL_Init(&LIS302DL_InitStruct);
	    
	  /* Set configuration of Internal High Pass Filter of LIS302DL*/
	  LIS302DL_InterruptStruct.Latch_Request = LIS302DL_INTERRUPTREQUEST_LATCHED;
	  LIS302DL_InterruptStruct.SingleClick_Axes = LIS302DL_CLICKINTERRUPT_Z_ENABLE;
	  LIS302DL_InterruptStruct.DoubleClick_Axes = LIS302DL_DOUBLECLICKINTERRUPT_Z_ENABLE;
	  LIS302DL_InterruptConfig(&LIS302DL_InterruptStruct);

	  /* Required delay for the MEMS Accelerometer: Turn-on time = 3/Output data Rate
	                                                             = 3/100 = 30ms */
	  Delay(30);
	  
	  /* Configure Interrupt control register: enable Click interrupt1 */
	  ctrl = 0x07;
	  LIS302DL_Write(&ctrl, LIS302DL_CTRL_REG3_ADDR, 1);
	  
	  /* Enable Interrupt generation on click/double click on Z axis */
	  ctrl = 0x70;
	  LIS302DL_Write(&ctrl, LIS302DL_CLICK_CFG_REG_ADDR, 1);
	  
	  /* Configure Click Threshold on X/Y axis (10 x 0.5g) */
	  ctrl = 0xAA;
	  LIS302DL_Write(&ctrl, LIS302DL_CLICK_THSY_X_REG_ADDR, 1);
	  
	  /* Configure Click Threshold on Z axis (10 x 0.5g) */
	  ctrl = 0x0A;
	  LIS302DL_Write(&ctrl, LIS302DL_CLICK_THSZ_REG_ADDR, 1);
	  
	  /* Configure Time Limit */
	  ctrl = 0x03;
	  LIS302DL_Write(&ctrl, LIS302DL_CLICK_TIMELIMIT_REG_ADDR, 1);
	    
	  /* Configure Latency */
	  ctrl = 0x7F;
	  LIS302DL_Write(&ctrl, LIS302DL_CLICK_LATENCY_REG_ADDR, 1);
	  
	  /* Configure Click Window */
	  ctrl = 0x7F;
	  LIS302DL_Write(&ctrl, LIS302DL_CLICK_WINDOW_REG_ADDR, 1);

	
	LIS302DL_Read(Buffer, LIS302DL_OUT_X_ADDR, 6);
	                  
	  XOffset = Buffer[0];
	  YOffset = Buffer[2];

	ThrottleLinkConfig();

	while (1)
	{
		uint8_t i;
		for(i = 0; i < 8; ++i)
		{
			if(varGetBit(Buffer[0],i))
			{
				GPIO_SetBits(GPIOD,(uint16_t)1<<i);
			}
			else
			{
				GPIO_ResetBits(GPIOD, (uint16_t)1<<i);
			}
		}
	}
}

void Delay(__IO uint32_t nTime)
{ 
  TimingDelay = nTime;

  while(TimingDelay != 0);
}

/**
  * @brief  Decrements the TimingDelay variable.
  * @param  None
  * @retval None
  */
void TimingDelay_Decrement(void)
{
  if (TimingDelay != 0x00)
  { 
    TimingDelay--;
  }
}

/**
  * @brief  MEMS accelerometre management of the timeout situation.
  * @param  None.
  * @retval None.
  */
uint32_t LIS302DL_TIMEOUT_UserCallback(void)
{
  /* MEMS Accelerometer Timeout error occured */
  while (1)
  {   
  }
}

void ThrottleLinkConfig(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	/* GPIOA Periph clock enable */
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOD, &GPIO_InitStructure);
}


#ifdef  USE_FULL_ASSERT

/**
 * @brief  Reports the name of the source file and the source line number
 *         where the assert_param error has occurred.
 * @param  file: pointer to the source file name
 * @param  line: assert_param error line source number
 * @retval None
 */
void assert_failed(uint8_t* file, uint32_t line)
{
	/* User can add his own implementation to report the file name and line number,
	 ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

	while (1)
	{}
}
#endif

/**
 * @}
 */

/**
 * @}
 */

/******************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE****/
